@@ -1,4 +1,4 @@
-// $Id: commandids.h,v 1.7 1998/09/15 22:06:15 jsixt Exp $
+// $Id: commandids.h,v 1.9 1998/11/07 18:34:58 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -24,6 +24,8 @@
 #define ID_PROGRAM_BREAK 25
 #define ID_PROGRAM_ARGS 26
 #define ID_PROGRAM_RUN_AGAIN 27
+#define ID_PROGRAM_ATTACH 28
+#define ID_PROGRAM_DETACH 29
 
 #define ID_BRKPT_SET 30
 #define ID_BRKPT_TEMP 31
@@ -48,8 +50,8 @@
 #define ID_WINDOW_INDEX_MASK 0xf
 
 // statusbar ids
-#define ID_STATUS_MSG 1
-#define ID_STATUS_BUSY 2
-#define ID_STATUS_ACTIVE 3
-#define ID_STATUS_LINENO 4
+#define ID_STATUS_MSG 191
+#define ID_STATUS_BUSY 192
+#define ID_STATUS_ACTIVE 193
+#define ID_STATUS_LINENO 194
 #endif // COMMANDIDS_H
