@@ -1,4 +1,4 @@
-// $Id: debugger.cpp,v 1.25 1998/09/26 15:21:43 jsixt Exp $
+// $Id: debugger.cpp,v 1.29 1998/11/07 18:34:59 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -9,6 +9,7 @@
 #include "updateui.h"
 #include "parsevar.h"
 #include "pgmargs.h"
+#include "procattach.h"
 #include "typetable.h"
 #include <qregexp.h>
 #include <qfileinf.h>
@@ -77,6 +78,9 @@ KDebugger::KDebugger(const char* name) :
 	m_menu(this, "menu"),
 	m_toolbar(this, "toolbar"),
 	m_statusbar(this, "statusbar"),
+	m_animationTimer(this),
+	m_animationCounter(0),
+	m_animationInterval(0),
 	m_mainPanner(this, "main_pane", KNewPanner::Vertical, KNewPanner::Percent, 60),
 	m_leftPanner(&m_mainPanner, "left_pane", KNewPanner::Horizontal, KNewPanner::Percent, 70),
 	m_rightPanner(&m_mainPanner, "right_pane", KNewPanner::Horizontal, KNewPanner::Percent, 50),
@@ -97,7 +101,6 @@ KDebugger::KDebugger(const char* name) :
 	m_watchV(&m_watches, 0),
 	m_watchH(0)
 {
-    m_statusBusy = i18n("busy");
     m_statusActive = i18n("active");
 
     m_gdbOutputAlloc = 4000;
@@ -126,7 +129,7 @@ KDebugger::KDebugger(const char* name) :
 	    SLOT(slotLocalsExpanding(KTreeViewItem*,bool&)));
 
     m_leftPanner.activate(&m_filesWindow, &m_btWindow);
-    connect(&m_btWindow, SIGNAL(selected(int)), SLOT(gotoFrame(int)));
+    connect(&m_btWindow, SIGNAL(highlighted(int)), SLOT(gotoFrame(int)));
     connect(&m_watchVariables, SIGNAL(expanding(KTreeViewItem*,bool&)),
 	    SLOT(slotWatchExpanding(KTreeViewItem*,bool&)));
 
@@ -153,6 +156,10 @@ KDebugger::KDebugger(const char* name) :
 
     m_filesWindow.setWindowMenu(&m_menuWindow);
     connect(&m_filesWindow.m_findDlg, SIGNAL(closed()), SLOT(updateUI()));
+    connect(&m_filesWindow, SIGNAL(toggleBreak(const QString&, int)),
+	    SLOT(slotToggleBreak(const QString&,int)));
+    connect(&m_filesWindow, SIGNAL(enadisBreak(const QString&, int)),
+	    SLOT(slotEnaDisBreak(const QString&,int)));
 
     m_bpTable.setCaption(i18n("Breakpoints"));
     connect(&m_bpTable, SIGNAL(closed()), SLOT(updateUI()));
@@ -162,7 +169,7 @@ KDebugger::KDebugger(const char* name) :
     // file/line updates
     connect(&m_filesWindow, SIGNAL(fileChanged()), SLOT(slotFileChanged()));
     connect(&m_filesWindow, SIGNAL(lineChanged()), SLOT(slotLineChanged()));
-    
+
     // debugger process
     connect(&m_gdb, SIGNAL(receivedStdout(KProcess*,char*,int)),
 	    SLOT(receiveOutput(KProcess*,char*,int)));
@@ -346,6 +353,19 @@ void KDebugger::menuCallback(int item)
 	    m_programRunning = true;
 	}
 	break;
+    case ID_PROGRAM_ATTACH:
+	if (isReady()) {
+	    ProcAttach dlg(this);
+	    dlg.setText(m_attachedPid);
+	    if (dlg.exec()) {
+		m_attachedPid = dlg.text();
+		TRACE("Attaching to " + m_attachedPid);
+		executeCmd(DCattach, "attach " + m_attachedPid);
+		m_programActive = true;
+		m_programRunning = true;
+	    }
+	}
+	break;
     case ID_PROGRAM_RUN_AGAIN:
 	if (isReady() && m_programActive && !m_programRunning) {
 	    executeCmd(DCrun, "run " + m_programArgs, true);
@@ -464,6 +484,17 @@ void KDebugger::updateUI()
     UpdateToolbarUI updateToolbar(&m_toolbar, this, SLOT(updateUIItem(UpdateUI*)),
 				  toolIds, sizeof(toolIds)/sizeof(toolIds[0]));
     updateToolbar.iterateToolbar();
+    // special update of animation
+    if (m_state == DSidle) {
+	stopAnimation();
+    } else {
+	/*
+	 * Slow animation while program is stopped (i.e. while variables
+	 * are displayed)
+	 */
+	bool slow = isReady() && m_programActive && !m_programRunning;
+	startAnimation(!slow);
+    }
 }
 
 void KDebugger::updateUIItem(UpdateUI* item)
@@ -485,6 +516,7 @@ void KDebugger::updateUIItem(UpdateUI* item)
     case ID_PROGRAM_RUN_AGAIN:
 	item->enable(isReady() && m_programActive && !m_programRunning);
 	break;
+    case ID_PROGRAM_ATTACH:
     case ID_PROGRAM_RUN:
 	item->enable(isReady());
 	break;
@@ -506,8 +538,6 @@ void KDebugger::updateUIItem(UpdateUI* item)
     }
     
     // update statusbar
-    m_statusbar.changeItem(m_state == DSidle ? "" : static_cast<const char*>(m_statusBusy),
-			   ID_STATUS_BUSY);
     m_statusbar.changeItem(m_programActive ? static_cast<const char*>(m_statusActive) : "",
 			   ID_STATUS_ACTIVE);
     // line number is updated in slotLineChanged
@@ -540,6 +570,7 @@ void KDebugger::initMenu()
     m_menuProgram.insertSeparator();
     m_menuProgram.insertItem(i18n("&Break"), ID_PROGRAM_BREAK);
     m_menuProgram.insertItem(i18n("Re&start"), ID_PROGRAM_RUN_AGAIN);
+    m_menuProgram.insertItem(i18n("A&ttach..."), ID_PROGRAM_ATTACH);
     m_menuProgram.insertSeparator();
     m_menuProgram.insertItem(i18n("&Arguments..."), ID_PROGRAM_ARGS);
     m_menuProgram.setAccel(Key_F5, ID_PROGRAM_RUN);
@@ -588,6 +619,8 @@ void KDebugger::initToolbar()
 {
     KIconLoader* loader = kapp->getIconLoader();
 
+    m_toolbar.insertButton(loader->loadIcon("execopen.xpm"),ID_FILE_EXECUTABLE, true,
+			   i18n("Executable"));
     m_toolbar.insertButton(loader->loadIcon("fileopen.xpm"),ID_FILE_OPEN, true,
 			   i18n("Open a source file"));
     m_toolbar.insertButton(loader->loadIcon("reload.xpm"),ID_FILE_RELOAD, true,
@@ -609,18 +642,42 @@ void KDebugger::initToolbar()
 			   i18n("Search"));
 
     connect(&m_toolbar, SIGNAL(clicked(int)), SLOT(menuCallback(int)));
+    
+    initAnimation();
 
-    m_statusbar.insertItem(m_statusBusy, ID_STATUS_BUSY);
     m_statusbar.insertItem(m_statusActive, ID_STATUS_ACTIVE);
     m_statusbar.insertItem(i18n("Line 00000"), ID_STATUS_LINENO);
     m_statusbar.insertItem("", ID_STATUS_MSG);	/* message pane */
 
     // reserve some translations
     i18n("Restart");
-    i18n("Executable");
     i18n("Core dump");
 }
 
+void KDebugger::initAnimation()
+{
+    QPixmap pixmap;
+    QString path = kapp->kde_datadir() + "/kfm/pics/";
+
+    pixmap.load(path + "/kde1.xpm");
+    
+    m_toolbar.insertButton(pixmap, ID_STATUS_BUSY);
+    m_toolbar.alignItemRight(ID_STATUS_BUSY, true);
+    
+    // Load animated logo
+    m_animation.setAutoDelete(true);
+    QString n;
+    for (int i = 1; i <= 9; i++) {
+	n.sprintf("/kde%d.xpm", i);
+	QPixmap* p = new QPixmap();
+	p->load(path + n);
+	if (!p->isNull()) {
+	    m_animation.append(p);
+	}
+    }
+    connect(&m_animationTimer, SIGNAL(timeout()), SLOT(slotAnimationTimeout()));
+}
+
 #ifdef WANT_THIS_PANE
 bool KDebugger::isThisPaneVisible()
 {
@@ -1069,7 +1126,6 @@ void KDebugger::receiveOutput(KProcess*, char* buffer, int buflen)
     if (m_activeCmd == 0 && m_state != DSinterrupted) {
 	// ignore the output
 	TRACE("ignoring stray output: " + QString(buffer, buflen+1));
-	ASSERT(buflen >= PROMPT_LEN && strncmp(buffer, PROMPT, PROMPT_LEN) == 0);
 	return;
     }
     ASSERT(m_state == DSrunning || m_state == DSrunningLow || m_state == DSinterrupted);
@@ -1127,6 +1183,13 @@ void KDebugger::receiveOutput(KProcess*, char* buffer, int buflen)
 	// empty buffer
 	m_gdbOutputLen = 0;
 	*m_gdbOutput = '\0';
+	// also clear delayed output if interrupted
+	if (m_state == DSinterrupted) {
+	    QString* delayed;
+	    while ((delayed = m_delayedOutput.dequeue()) != 0) {
+		delete delayed;
+	    }
+	}
 
 	/*
 	 * We parsed some output successfully. Unless there's more delayed
@@ -1271,6 +1334,7 @@ void KDebugger::parse(CmdQueueItem* cmd)
     case DCprint:
 	handlePrint(cmd);
 	break;
+    case DCattach:
     case DCrun:
     case DCcont:
     case DCstep:
@@ -1346,9 +1410,11 @@ void KDebugger::handleRunCommands()
     do {
 	start++;			/* skip '\n' */
 
-	if (strncmp(start, "Program ", 7) == 0) {
+	if (strncmp(start, "Program ", 8) == 0 ||
+	    strncmp(start, "ptrace: ", 8) == 0) {
 	    if (strncmp(start, "Program exited", 14) == 0 ||
-		strncmp(start, "Program terminated", 18) == 0)
+		strncmp(start, "Program terminated", 18) == 0 ||
+		strncmp(start, "ptrace: ", 8) == 0)
 	    {
 		m_programActive = false;
 	    }
@@ -1888,7 +1954,7 @@ void KDebugger::dereferencePointer(ExprWnd* wnd, VarTree* exprItem,
     QString queueExpr = "print *(" + expr + ")";
     CmdQueueItem* cmd;
     if (immediate) {
-	cmd = executeCmd(DCprint, queueExpr);
+	cmd = queueCmd(DCprint, queueExpr, QMoverrideMoreEqual);
     } else {
 	cmd = queueCmd(DCprint, queueExpr, QMoverride);
     }
@@ -2204,3 +2270,47 @@ void KDebugger::updateLineStatus(int lineNo)
 	m_statusbar.changeItem(strLine, ID_STATUS_LINENO);
     }
 }
+
+void KDebugger::slotAnimationTimeout()
+{
+    m_animationCounter++;
+    if (m_animationCounter == m_animation.count())
+	m_animationCounter = 0;
+    m_toolbar.setButtonPixmap(ID_STATUS_BUSY,
+			      *m_animation.at(m_animationCounter));
+}
+
+void KDebugger::startAnimation(bool fast)
+{
+    int interval = fast ? 50 : 150;
+    if (!m_animationTimer.isActive()) {
+	m_animationTimer.start(interval);
+    } else if (m_animationInterval != interval) {
+	m_animationTimer.changeInterval(interval);
+    }
+    m_animationInterval = interval;
+}
+
+void KDebugger::stopAnimation()
+{
+    if (m_animationTimer.isActive()) {
+	m_animationTimer.stop();
+	m_animationInterval = 0;
+    }
+}
+
+void KDebugger::slotToggleBreak(const QString& fileName, int lineNo)
+{
+    // lineNo is zero-based
+    if (isReady()) {
+	m_bpTable.doBreakpoint(fileName, lineNo, false);
+    }
+}
+
+void KDebugger::slotEnaDisBreak(const QString& fileName, int lineNo)
+{
+    // lineNo is zero-based
+    if (isReady()) {
+	m_bpTable.doEnableDisableBreakpoint(fileName, lineNo);
+    }
+}
