@@ -1,4 +1,4 @@
-// $Id: debugger.h,v 1.15 1998/09/20 16:37:00 jsixt Exp $
+// $Id: debugger.h,v 1.19 1998/11/07 18:35:00 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -59,6 +59,7 @@ public:
 	DCtty,
 	DCexecutable,
 	DCcorefile,
+	DCattach,
 	DCinfolinemain,
 	DCinfolocals,
 	DCbt,
@@ -187,6 +188,7 @@ protected:
     bool m_programRunning;		/* is the program executing (not stopped)? */
     QString m_executable;
     QString m_corefile;
+    QString m_attachedPid;		/* user input of attaching to pid */
     QString m_programArgs;
     KSimpleConfig* m_programConfig;	/* program-specific settings (brkpts etc) */
     void saveProgramSettings();
@@ -222,6 +224,9 @@ public slots:
     void slotWatchExpanding(KTreeViewItem*, bool&);
     void slotFileChanged();
     void slotLineChanged();
+    void slotAnimationTimeout();
+    void slotToggleBreak(const QString&, int);
+    void slotEnaDisBreak(const QString&, int);
     /*
      * Unless WANT_THIS_PANE is defined, the following slots are unused.
      * Removing them with #ifdef ... #endif doesn't work because moc
@@ -242,8 +247,12 @@ protected:
     KToolBar m_toolbar;
     KStatusBar m_statusbar;
     // statusbar texts
-    QString m_statusBusy;
     QString m_statusActive;
+    // animated buttons
+    QList<QPixmap> m_animation;
+    QTimer m_animationTimer;
+    uint m_animationCounter;
+    int m_animationInterval;
     
     // view windows
     KNewPanner m_mainPanner;
@@ -279,6 +288,9 @@ protected:
 protected:
     void initMenu();
     void initToolbar();
+    void initAnimation();
+    void startAnimation(bool fast);
+    void stopAnimation();
 
     friend class BreakpointTable;
 };
