@@ -1,4 +1,4 @@
-// $Id: pgmargs.h,v 1.1 1998/05/03 14:58:53 jsixt Exp $
+// $Id: pgmargs.h,v 1.2 1998/11/07 18:31:38 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -14,7 +14,6 @@
 
 class PgmArgs : public QDialog
 {
-    Q_OBJECT
 public:
     PgmArgs(QWidget* parent, const char* pgm);
     virtual ~PgmArgs();
@@ -22,12 +21,6 @@ public:
     void setText(const char* text) { m_programArgs.setText(text); }
     const char* text() const { return m_programArgs.text(); }
 
-public slots:
-
-protected slots:
-//    virtual void accept();
-//    virtual void reject();
-
 protected:
     QLabel m_label;
     QLineEdit m_programArgs;
@@ -35,8 +28,6 @@ protected:
     QPushButton m_buttonCancel;
     QVBoxLayout m_layout;
     QHBoxLayout m_buttons;
-
-//    virtual void resizeEvent(QResizeEvent* ev);
 };
 
 #endif // PgmArgs_included
