@@ -1,4 +1,4 @@
-// $Id: winstack.h,v 1.11 1998/09/16 20:18:34 jsixt Exp $
+// $Id: winstack.h,v 1.12 1998/10/11 20:08:05 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -42,8 +42,13 @@ protected:
     virtual int textCol() const;
     virtual int cellWidth(int col);
     virtual void paintCell(QPainter* p, int row, int col);
+    virtual void mouseReleaseEvent(QMouseEvent* ev);
     void updateLineItem(int i);
 
+signals:
+    void clickedLeft(const QString&, int);
+    void clickedMid(const QString&, int);
+
 protected:
     QString m_fileName;
     enum LineItem { liPC = 1, liPCup = 2,
@@ -110,12 +115,16 @@ public:
 signals:
     void fileChanged();
     void lineChanged();
+    void toggleBreak(const QString&, int);
+    void enadisBreak(const QString&, int);
 
 public slots:
     virtual void menuCallback(int item);
     virtual void slotLineChanged();
     virtual void slotFindForward();
     virtual void slotFindBackward();
+    virtual void slotToggleBreak(const QString&,int);
+    virtual void slotEnaDisBreak(const QString&,int);
 
 protected:
     bool activateFI(QFileInfo& fi, int lineNo);
